<?php

namespace Drupal\commerce_ticketing\Mail;

use Drupal\commerce_ticketing\CommerceTicketInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\entity_print\Plugin\EntityPrintPluginManagerInterface;
use Drupal\entity_print\PrintBuilderInterface;

class TicketReceiptMail {

  use StringTranslationTrait;

  /**
   * Logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The plugin manager for our Print engines.
   *
   * @var \Drupal\entity_print\Plugin\EntityPrintPluginManagerInterface
   */
  protected $pluginManager;

  /**
   * The Print builder.
   *
   * @var \Drupal\entity_print\PrintBuilderInterface
   */
  protected $printBuilder;

  /**
   * The object renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * TicketReceiptMail constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   Logger.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   Mail manager service.
   * @param \Drupal\entity_print\Plugin\EntityPrintPluginManagerInterface $plugin_manager
   *   Entity Print plugin manager.
   * @param \Drupal\entity_print\PrintBuilderInterface $print_builder
   *   Entity Print builder.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The object renderer.
   */
  public function __construct(LoggerChannelInterface $logger, EntityTypeManagerInterface $entity_type_manager, MailManagerInterface $mail_manager, EntityPrintPluginManagerInterface $plugin_manager, PrintBuilderInterface $print_builder, RendererInterface $renderer) {
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger;
    $this->mailManager = $mail_manager;
    $this->pluginManager = $plugin_manager;
    $this->printBuilder = $print_builder;
    $this->renderer = $renderer;
  }

  /**
   * Sends ticket receipt by email and attaches the PDF if available.
   */
  public function send(CommerceTicketInterface $ticket, $to = NULL, $bcc = NULL) {
    try {
      $order = $ticket->getOrder();
      // If there's no recipient set, fallback to the ticket owner and then the
      // order owner.
      $to = $to ?? $ticket->getOwner()->getEmail() ?? $ticket->getOrder();
      if (!$to) {
        return FALSE;
      }
      $subject = $this->t('Ticket for your order #@number', ['@number' => $order->getOrderNumber()]);
      $body = $this->entityTypeManager->getViewBuilder('commerce_ticket')->view($ticket, 'receipt');
      $params = [
        'subject' => $subject,
        'from' => $order->getStore()->getEmail(),
        'bcc' => $bcc,
        'order' => $order,
        'ticket' => $ticket,
        'body' => $this->renderer->renderRoot($body),
      ];

      $print_engine = $this->pluginManager->createSelectedInstance('pdf');
      ob_start();
      $this->printBuilder->deliverPrintable([$ticket], $print_engine, TRUE);
      $contents = ob_get_contents();
      ob_end_clean();
      $params['attachment'] = [
        'filename' => $ticket->uuid() . '.pdf',
        'filecontent' => $contents,
        'filemime' => 'application/pdf',
      ];

      // Default site langcode, if the ticket owner is not anonymous, use that
      // instead.
      $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
      if ($ticket->getOwner()->isAuthenticated()) {
        $langcode = $ticket->getOwner()->getPreferredLangcode();
      }

      $result = $this->mailManager->mail('commerce_ticketing', 'ticket_receipt', $to, $langcode, $params, NULL, TRUE);
      if ($result['result']) {
        return TRUE;
      }
      else {
        $this->logger->warning('Email for ticket %ticket was not sent to: %to', ['%ticket' => $ticket->uuid(), '%to' => $to]);
        return FALSE;
      }
    }
    catch (\Exception $e) {
      watchdog_exception('commerce_ticketing', $e);
      return FALSE;
    }
  }

}
